
/**
 * @imports
 */
import _isString from '@web-native-js/commons/js/isString.js';
import _isArray from '@web-native-js/commons/js/isArray.js';
import _isObject from '@web-native-js/commons/js/isObject.js';
import _toCamel from '@web-native-js/commons/str/toCamel.js';
import _objFrom from '@web-native-js/commons/obj/from.js';
import _each from '@web-native-js/commons/obj/each.js';
import _arrFrom from '@web-native-js/commons/arr/from.js';
import _difference from '@web-native-js/commons/arr/difference.js';
import _exclude from '@web-native-js/commons/arr/exclude.js';
import _unique from '@web-native-js/commons/arr/unique.js';
import _intersect from '@web-native-js/commons/arr/intersect.js';

/**
 * Gets an attribute or a list of attributes,
 * or sets an attribute or a list of attributes.
 *
 * @param DOMElement			el 
 * @param string|array|object	requestOrPayload
 * @param string|bool|void		valOrMutation
 * @param bool|void				subValMutation
 *
 * @return mixed
 */
export default function(el, requestOrPayload, valOrMutation = null, subValMutation = null) {
	if (arguments.length === 2) {
		if (_isString(requestOrPayload)) {
			return el.getAttribute(requestOrPayload);
		}
		if (_isArray(requestOrPayload)) {
			var vals = {};
			requestOrPayload.forEach(request => {
				vals[request] = el.getAttribute(request);
			});
			return vals;
		}
	}
	var payload = requestOrPayload;
	if (!_isObject(payload)) {
		payload = _objFrom(requestOrPayload, valOrMutation);
	} else {
		subValMutation = valOrMutation;
	}
	_each(payload, (name, valOrMutation) => {
		if (arguments.length > 3 || (_isObject(requestOrPayload) && arguments.length > 2)) {
			var currentVal = el.getAttribute(name);
			var currentValArray = currentVal ? currentVal.split(' ').map(val => val.trim()).filter(a => a) : [];
			// -----------------------
			// Add or remove
			// -----------------------
			var values = _isString(valOrMutation) ? valOrMutation.trim().split(' ').map(val => val.trim()).filter(a => a) : _arrFrom(valOrMutation);
			if (!subValMutation && _intersect(currentValArray, values).length) {
				// Add...
				el.setAttribute(name, _exclude(currentValArray, ...values).join(' '));
			} else if (subValMutation && _difference(currentValArray, values).length) {
				// Remove...
				el.setAttribute(name, _unique(currentValArray.concat(values)).join(' '));
			}
		} else {
			if (valOrMutation === false) {
				el.removeAttribute(name);
			} else {
				el.setAttribute(name, valOrMutation === true ? 'true' : valOrMutation);
			}
		}
	});
	return el;
};
