
/**
 * @imports
 */
import _isString from '@web-native-js/commons/js/isString.js';
import _arrFrom from '@web-native-js/commons/arr/from.js';

/**
 * Observes when the given elements or selectors are added or removed
 * from the given context.
 *
 * @param array|HTMLElement|string	els
 * @param function					callback
 * @param int						connectedState
 * @param HTMLElement|string		context
 * @param bool						observeIndirectMutation
 *
 * @return MutationObserver
 */
export default function(els, callback, connectedState = null, context = null, observeIndirectMutation = true) {
	var search = (el, nodeListArray) => {
		if (_isString(el)) {
			// Is directly mutated...
			var matches = nodeListArray.filter(node => node.matches(el));
			if (matches.length) {
				return matches;
			}
			// Is contextly mutated...
			if (observeIndirectMutation) {
				matches = nodeListArray
					.reduce((collection, node) => collection.concat(_arrFrom(node.querySelectorAll(el))), []);
				if (matches.length) {
					return matches;
				}
			}
		} else {
			// Is directly mutated...
			if (nodeListArray.includes(el)) {
				return el;
			}
			// Is contextly mutated...
			if (observeIndirectMutation && nodeListArray.length) {
				var parentNode = el;
				while(!success && (parentNode = parentNode.parentNode)) {
					if (nodeListArray.includes(parentNode)) {
						return el;
					}
				}
			}
		}
	};
	var mo = new MutationObserver(mutations => {
		if (connectedState !== 0) {
			var matchedAddedNodes = [];
			_arrFrom(els, false/*castObject*/).forEach(el => {
				if (_isString(el)) {
					matchedAddedNodes = mutations
						.reduce((matches, mut) => matches.concat(search(el, _arrFrom(mut.addedNodes)), matchedAddedNodes));
				} else {
					var matchedAsAddedNode = mutations
						.reduce((match, mut) => match || search(el, _arrFrom(mut.addedNodes)), null);
					if (matchedAsAddedNode) {
						matchedAddedNodes.push(matchedAsAddedNode);
					}
				}
			});
			if (matchedAddedNodes.length) {
				callback(1, ...matchedAddedNodes);
			}
		}
		if (connectedState !== 1) {
			var matchedRemovedNodes = [];
			_arrFrom(els, false/*castObject*/).forEach(el => {
				if (_isString(el)) {
					matchedRemovedNodes = mutations
						.reduce((matches, mut) => matches.concat(search(el, _arrFrom(mut.removedNodes)), matchedRemovedNodes));
				} else {
					var matchedAsRemovedNode = mutations
						.reduce((match, mut) => match || search(el, _arrFrom(mut.removedNodes)), null);
					if (matchedAsRemovedNode) {
						matchedRemovedNodes.push(matchedAsRemovedNode);
					}
				}
			});
			if (matchedRemovedNodes.length) {
				callback(0, ...matchedRemovedNodes);
			}
		}
	});
	mo.observe(context || document.body, {childList:true, subtree:true});
	return mo;
};