
/**
 * @imports
 */
import _beforeLast from '@web-native-js/commons/str/beforeLast.js';
import _arrFrom from '@web-native-js/commons/arr/from.js';
import _each from '@web-native-js/commons/obj/each.js';
import * as modules from './modules.js';

/**
 * Create a prototyper.
 *
 * @param object ob
 * @param object params
 * 
 * @return void
 */
export function extend(target, params = {}) {
    var getReturn = (instance, ret) => {
        if (params.factory) {
            if (ret instanceof Promise) {
                return ret.then(_ret => _ret === instance.el ? instance : _ret);
            }
            return ret === instance.el ? instance : ret;
        }
        return ret;
    };
    _each(modules, (name, fn) => {
        if (name === 'el' || name === 'select' || name === 'selectAll') {
            target[name] = function(input, ...args) {
                var result = fn(input);
                if (params.factory) {
                    if (name === 'selectAll') {
                        result = _arrFrom(result).map(_el => params.factory(_el, ...args));
                    } else {
                        result = params.factory(result, ...args);
                    }
                }
                return result;
            };
        } else {
            target.prototype[name] = function(...args) {
                return getReturn(this, fn(this.el || document.createElement('div'), ...args));
            };
            // Create short form
            if (name.endsWith('Sync') || name.endsWith('Async')) {
                if (params.syncIsDefault && name.endsWith('Sync')) {
                    name = _beforeLast(name, 'Sync');
                } else if (!params.syncIsDefault && name.endsWith('Async')) {
                    name = _beforeLast(name, 'Async');
                }
                // Add to prototype now
                target.prototype[name] = function(...args) {
                    return getReturn(this, fn(this.el || document.createElement('div'), ...args));
                };
            }
        }
    });
};

/**
 * The Firedom Function.
 *
 * @param HTMLElement el
 * 
 * @return void
 */
export default function Firedom(el) {
    if (!(this instanceof Firedom)) {
        return new Firedom(el);
    }
    var _this = this;
    _this.el = modules.el(el);
};